@@ -54,15 +54,14 @@
 #include "qtadsappctx.h"
 
 
-// This contains the filename of the game to run.
-QString gameFileName;
-
-// This contains the filename of the most recently run game.
-QString lastGameFileName;
-
-
 int main( int argc, char* argv[] )
 {
+	// This contains the filename of the game to run.
+	QString gameFileName;
+
+	// This contains the filename of the most recently run game.
+	QString lastGameFileName;
+
 	const QString QTDIR(std::getenv("QTDIR"));
 
 	// The application object.
@@ -105,18 +104,19 @@ int main( int argc, char* argv[] )
 	}
 
 	if (argc == 2) {
-		if (QFile::exists(argv[1])) {
-			::gameFileName = argv[1];
-		} else if (QFile::exists(QString(argv[1]) + ".gam")) {
-			::gameFileName = QString(argv[1]) + ".gam";
-		} else if (QFile::exists(QString(argv[1]) + ".t3")) {
-			::gameFileName = QString(argv[1]) + ".t3";
+		if (QFile::exists(QString::fromLocal8Bit(argv[1]))) {
+			gameFileName = QString::fromLocal8Bit(argv[1]);
+		//} else if (QFile::exists(QString(argv[1]) + ".gam")) {
+		} else if (QFile::exists(QString::fromLocal8Bit(argv[1]) + ".gam")) {
+			gameFileName = QString::fromLocal8Bit(argv[1]) + ".gam";
+		} else if (QFile::exists(QString::fromLocal8Bit(argv[1]) + ".t3")) {
+			gameFileName = QString::fromLocal8Bit(argv[1]) + ".t3";
 		} else {
 			std::cerr << "File `" << argv[1] << "' not found." << std::endl;
 			return 2;
 		}
 	} else {
-		::gameFileName = QString::null;
+		gameFileName = QString::null;
 	}
 
 	// Construct the "Welcome" message.
@@ -164,7 +164,7 @@ int main( int argc, char* argv[] )
 
 	bool firstRun = true;
 	while (not QTadsIO::quitApp) {
-		if (::gameFileName.isEmpty()) {
+		if (gameFileName.isEmpty()) {
 			if (QTadsIO::gameWindow().text().isEmpty() and showWelcomeMsg) {
 				// Show the "Welcome" message.
 				QTadsIO::gameWindow().setTextFormat(Qt::RichText);
@@ -174,7 +174,7 @@ int main( int argc, char* argv[] )
 			}
 			// Prompt for a game file.
 			if (not (firstRun and not QTadsIO::settings().promptAtStartup())) {
-				::gameFileName = QTadsIO::openFile(QString::null,
+				gameFileName = QTadsIO::openFile(QString::null,
 								   QObject::tr("Tads Games")
 								   + " (*.gam *.Gam *.GAM *.t3 *.T3)"
 								   ";;"
@@ -186,24 +186,24 @@ int main( int argc, char* argv[] )
 		}
 		firstRun = false;
 
-		if (not ::gameFileName.isEmpty()) {
+		if (not gameFileName.isEmpty()) {
 
 			// Convert the filename to an absolute path...
-			::gameFileName = QFileInfo(::gameFileName).absFilePath();
+			gameFileName = QFileInfo(gameFileName).absFilePath();
 			// ...and change to the game's directory.  This
 			// makes it simpler to access files that the
 			// game might request (for example registration
 			// key-files, like regkey.txt with "Losing Your
 			// Grip").
-			if (not QDir::setCurrent(QFileInfo(::gameFileName).dirPath(true))) {
+			if (not QDir::setCurrent(QFileInfo(gameFileName).dirPath(true))) {
 				qtadsWarning(QObject::tr("Could not change current directory")
-					     + " (" + QFileInfo(::gameFileName).dirPath(true)
+					     + " (" + QFileInfo(gameFileName).dirPath(true)
 					     + ")");
 			}
 
 			os_init(&appArgc, appArgv, 0, 0, 0);
 			QTadsIO::gameRunning = true;
-			QTadsIO::currentGame = ::gameFileName;
+			QTadsIO::currentGame = gameFileName;
 			QTadsIO::enableCommandActions(true);
 			if (QTadsIO::gameWindow().isReadOnly()) {
 				QTadsIO::gameWindow().setReadOnly(false);
@@ -211,7 +211,7 @@ int main( int argc, char* argv[] )
 			QTadsIO::reset();
 
 			// Add the filename to the "Recent Games" menu...
-			QTadsIO::settings().addRecentGame(::gameFileName);
+			QTadsIO::settings().addRecentGame(gameFileName);
 			// ...and update it.
 			QTadsIO::mainWindow().updateRecentGamesList();
 
@@ -220,26 +220,28 @@ int main( int argc, char* argv[] )
 
 			QTadsIO::mainWindow().startTimer();
 			int ret;
-			if (vm_get_game_type(::gameFileName.ascii(), 0, 0, 0, 0)
+			if (vm_get_game_type(gameFileName.local8Bit(), 0, 0, 0, 0)
 			    == VM_GGT_TADS2)
 			{
 				QTadsIO::t3Mode(false);
-				std::strncpy(appArgv[1], ::gameFileName.ascii(), 512);
+				std::strncpy(appArgv[1], gameFileName.local8Bit(), 512);
+				// Initialize CString codec to the current locale. Some Tads 2 games don't
+				// use character mapping files, so at least the correct codec will be used
+				// if the user's system locale is using the same codec as the game file.
+				QTextCodec::setCodecForCStrings(QTextCodec::codecForLocale());
 				ret = trdmain(appArgc, appArgv, &appctx, "sav");
-				// Reset the text codec back to the default
-				// (in case the game was using a character
-				// mapping file).
+				// Reset the text codec back to the default.
 				QTextCodec::setCodecForCStrings(0);
 			} else {
 				QTadsIO::t3Mode(true);
-				ret = vm_run_image(&clientifc, ::gameFileName.ascii(),
+				ret = vm_run_image(&clientifc, gameFileName.local8Bit(),
 						   hostifc, appArgv, appArgc, 0, false, 0, 0, false,
 						   false, 0, 0, 0, 0);
 			}
 
 			QTadsIO::mainWindow().setCaption("QTads");
 			QTadsIO::mainWindow().stopTimer();
-			::lastGameFileName = ::gameFileName;
+			lastGameFileName = gameFileName;
 			showWelcomeMsg = false;
 			QTadsIO::mainWindow().gameInfoAction->setEnabled(false);
 			QTadsIO::enableCommandActions(false);
@@ -269,17 +271,17 @@ int main( int argc, char* argv[] )
 				break;
 			}
 			if (QTadsIO::restartGame) {
-				if (::gameFileName.isEmpty()) {
-					::gameFileName = ::lastGameFileName;
+				if (gameFileName.isEmpty()) {
+					gameFileName = lastGameFileName;
 				}
 				QTadsIO::restartGame = false;
 				break;
 			}
 			if (QTadsIO::openNewGame) {
 				if (QTadsIO::newGameToOpen.isEmpty()) {
-					::gameFileName = QString::null;
+					gameFileName = QString::null;
 				} else {
-					::gameFileName = QTadsIO::newGameToOpen;
+					gameFileName = QTadsIO::newGameToOpen;
 					QTadsIO::newGameToOpen = QString::null;
 				}
 				QTadsIO::openNewGame = false;
