@@ -305,7 +305,7 @@ osfacc( const char* fname )
 {
 	Q_ASSERT(fname != 0);
 
-	return QFileInfo(fname).exists() ? 0 : 1;
+	return QFileInfo(QString::fromLocal8Bit(fname)).exists() ? 0 : 1;
 }
 
 
@@ -397,10 +397,10 @@ os_locate( const char* fname, int /*flen*/, const char* /*arg0*/, char* buf, siz
 	Q_ASSERT(buf != 0);
 
 	// Current directory.
-	if (QFile::exists(fname)) {
-		Q_ASSERT(bufsiz > std::strlen(QFileInfo(fname).absFilePath().ascii()));
+	if (QFile::exists(QString::fromLocal8Bit(fname))) {
+		Q_ASSERT(bufsiz > std::strlen(QFileInfo(fname).absFilePath().local8Bit()));
 
-		std::strncpy(buf, QFileInfo(fname).absFilePath().ascii(), bufsiz);
+		std::strncpy(buf, QFileInfo(fname).absFilePath().local8Bit(), bufsiz);
 		buf[bufsiz - 1] = '\0';
 		return 1;
 	}
@@ -438,7 +438,7 @@ osfdel_temp( const char* fname )
 {
 	Q_ASSERT(fname != 0);
 
-	if (fname[0] == '\0' or QFile::remove(fname)) {
+	if (fname[0] == '\0' or QFile::remove(QString::fromLocal8Bit(fname))) {
 		// If fname was empty, it has been already deleted
 		// automatically by fclose().  If fname was not empty,
 		// QFile::remove has taken care of deleting the file.
@@ -462,7 +462,7 @@ os_defext( char* fn, const char* ext )
 	Q_ASSERT(fn != 0);
 	Q_ASSERT(ext != 0);
 
-	if (QFileInfo(fn).extension().isEmpty()) {
+	if (QFileInfo(QString::fromLocal8Bit(fn)).extension().isEmpty()) {
 		os_addext(fn, ext);
 	}
 }
@@ -491,7 +491,7 @@ os_remext( char* fn )
 {
 	Q_ASSERT(fn != 0);
 
-	QFileInfo file(fn);
+	QFileInfo file(QString::fromLocal8Bit(fn));
 
 	if (file.extension().isEmpty()) {
 		return;
@@ -538,7 +538,7 @@ os_build_full_path( char* fullpathbuf, size_t fullpathbuflen, const char* path,
 	Q_ASSERT(path != 0);
 	Q_ASSERT(filename != 0);
 
-	std::strncpy(fullpathbuf, QFileInfo(QDir(path), filename).filePath().ascii(),
+	std::strncpy(fullpathbuf, QFileInfo(QDir(QString::fromLocal8Bit(path)), QString::fromLocal8Bit(filename)).filePath().local8Bit(),
 		     fullpathbuflen);
 	fullpathbuf[fullpathbuflen - 1] = '\0';
 }
@@ -812,25 +812,25 @@ os_askfile( const char* prompt, char* fname_buf, int fname_buf_len, int prompt_t
 
 	switch (file_type) {
 	  case OSFTGAME:
-		filter = QObject::tr("TADS 2 Games (*.gam; *.Gam; *.GAM)");
+		filter = QObject::tr("TADS 2 Games (*.gam *.Gam *.GAM)");
 		break;
 	  case OSFTSAVE:
-		filter = QObject::tr("TADS 2 Saved Games (*.sav; *.Sav; *.SAV)");
+		filter = QObject::tr("TADS 2 Saved Games (*.sav *.Sav *.SAV)");
 		break;
 	  case OSFTLOG:
-		filter = QObject::tr("Game Transcripts (*.txt; *.Txt; *.TXT)");
+		filter = QObject::tr("Game Transcripts (*.txt *.Txt *.TXT)");
 		break;
 	  case OSFTT3IMG:
 #ifndef HTMLQT
 		Q_ASSERT(QTadsIO::t3Mode());
 #endif
-		filter = QObject::tr("TADS 3 Games (*.t3; *.T3)");
+		filter = QObject::tr("TADS 3 Games (*.t3 *.T3)");
 		break;
 	  case OSFTT3SAV:
 #ifndef HTMLQT
 		Q_ASSERT(QTadsIO::t3Mode());
 #endif
-		filter = QObject::tr("TADS 3 Saved Games (*.t3v; *.T3v; *.T3V)");
+		filter = QObject::tr("TADS 3 Saved Games (*.t3v *.T3v *.T3V)");
 		ext = "t3v";
 		break;
 	}
@@ -864,7 +864,7 @@ os_askfile( const char* prompt, char* fname_buf, int fname_buf_len, int prompt_t
 
 	Q_ASSERT(fname_buf_len > static_cast<int>(std::strlen(res.ascii())));
 
-	std::strncpy(fname_buf, res.ascii(), fname_buf_len);
+	std::strncpy(fname_buf, res.local8Bit(), fname_buf_len);
 	fname_buf[fname_buf_len - 1] = '\0';
 	if (not ext.isEmpty()) {
 		// Since `ext' is non-empty, an extension should be
@@ -1356,8 +1356,8 @@ os_gen_charmap_filename( char* filename, char* internal_id, char* /*argv0*/ )
 #ifndef HTMLQT
 	Q_ASSERT(filename != 0);
 
-	std::strncpy(filename, QString(QString(::CHARMAP_INST_DIR) + "/"
-		     + QString(internal_id).lower() + ".tcp").latin1(), OSFNMAX);
+	std::strncpy(filename, QString(QString::fromLocal8Bit(::CHARMAP_INST_DIR) + "/"
+		     + QString(internal_id).lower() + ".tcp").local8Bit(), OSFNMAX);
 	filename[OSFNMAX - 1] = '\0';
 #else
 	filename[0] = '\0';
