@@ -101,7 +101,7 @@ os_get_exe_filename( char* buf, size_t buflen, const char* argv0 )
  * TODO: Implement it.
  */
 void
-os_get_special_path( char* buf, size_t /*buflen*/, const char* /*argv0*/, int id )
+os_get_special_path( char* /*buf*/, size_t /*buflen*/, const char* /*argv0*/, int /*id*/ )
 {
 	/*
 	switch (id) {
