@@ -54,12 +54,6 @@ QTadsGameWindow::QTadsGameWindow( QWidget* parent, const char* name )
 
 	// *Always* wrap the text, no matter how (but wrapping at word
 	// boundaries will be preferred).
-	//
-	// TODO: Send a bug-report to Trolltech: setting
-	// AtWordOrDocumentBoundary works when running QTads with the
-	// same Qt version it has been compiled with, but doesn't work
-	// when running it using a different version.
-	//
 	QTextEdit::setWordWrap(QTextEdit::WidgetWidth);
 	QTextEdit::setWrapPolicy(QTextEdit::AtWordOrDocumentBoundary);
 
@@ -68,8 +62,6 @@ QTadsGameWindow::QTadsGameWindow( QWidget* parent, const char* name )
 	// We only use plain text mode and format the text using
 	// methods, not HTML-tags.
 	QTextEdit::setTextFormat(Qt::PlainText);
-	// This style displays best.
-	QTextEdit::setFrameStyle(QFrame::Box | QFrame::Plain);
 	// Every time the user clicks somewhere in the window, call our
 	// click handler.
 	connect(this, SIGNAL(clicked(int,int)), SLOT(clickHandler(int,int)));
