@@ -26,6 +26,7 @@
 #include <qfont.h>
 #include <qaction.h>
 #include <qrect.h>
+#include <qfileinfo.h>
 
 #include "qtadsio.h"
 #include "qtadsmainwindow.h"
@@ -208,9 +209,18 @@ QTadsSettings::QTadsSettings()
 	this->fRecentGames.reserve(this->fRecentGamesCapacity);
 	this->fRecentGamesCount = sett.readNumEntry(RECENT_KEY + "Count");
 	this->fRecentGames.resize(this->fRecentGamesCount);
+	int j = 0;
 	for (int i = 0; i < this->fRecentGamesCount; ++i) {
-		this->fRecentGames[i] = sett.readEntry(RECENT_KEY + QString::number(i));
+		QFileInfo file;
+		file.setFile(sett.readEntry(RECENT_KEY + QString::number(i)));
+		// Ignore the entry if the file is not accessible.
+		if (file.exists() and (file.isFile() or file.isSymLink()) and file.isReadable()) {
+			this->fRecentGames[j++] = file.filePath();
+		}
 	}
+	// Adapt the list due to possible non-accessible files.
+	this->fRecentGamesCount = j;
+	this->fRecentGames.resize(j);
 
 	// Load some miscellaneous settings.
 	//
