@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Nikos Chantziaras.
+/* Copyright (C) 2004 Nikos Chantziaras.
  *
  * This file is part of the QTads program.  This program is free
  * software; you can redistribute it and/or modify it under the
@@ -20,69 +20,30 @@
 /* Host interface.  Every application containing the VM must provide
  * the VM with an implementation of this interface.
  *
- * This implementation does nothing more than the simple stdio-based
- * one from the portable Tads 3 sources (tads3/vmhostsi.h), but this
- * might change in the future.
+ * This implementation does little more than the simple stdio-based one
+ * from the portable Tads 3 sources (tads3/vmhostsi.h).
  */
 
 #ifndef QTADSVMHOSTIFC_H
 #define QTADSVMHOSTIFC_H
 
-#include "resload.h"
-#include "vmhosttx.h"
+#include "config.h"
 
-class QTadsCVmHostIfc: public CVmHostIfcText {
-  protected:
-	// Character mapping file resource loader.
-	CResLoader* cmap_loader_;
+#include "vmhostsi.h"
 
-	// Current I/O safety level.
-	int io_safety_;
+#include "qtadsio.h"
+#include "qtadssettings.h"
 
+class QTadsCVmHostIfc: public CVmHostIfcStdio {
   public:
-	QTadsCVmHostIfc( const char* argv0 );
+	QTadsCVmHostIfc( const char *argv0 )
+	: CVmHostIfcStdio(argv0)
+	{}
 
-	virtual
-	~QTadsCVmHostIfc()
-	{
-		// Delete our character map resource loader.
-		delete cmap_loader_;
-	}
-
-	// Get the I/O safety level.
-	// We'll allow any file operations.
 	virtual int
 	get_io_safety()
 	{
-		return io_safety_;
-	}
-
-	// Set I/O safety level.
-	virtual void
-	set_io_safety( int level )
-	{
-		io_safety_ = level;
-	}
-
-	// Get the resource loader.
-	virtual CResLoader*
-	get_cmap_res_loader()
-	{
-		return cmap_loader_;
-	}
-
-	// Get the resource path.
-	virtual const char*
-	get_res_path()
-	{
-		return 0;
-	}
-
-	// Get an image file name.
-	virtual vmhost_gin_t
-	get_image_name( char*, size_t )
-	{
-		return VMHOST_GIN_IGNORED;
+		return QTadsIO::settings().fileIoSafetyLevel();
 	}
 };
 
