@@ -1344,8 +1344,12 @@ resume_from_error:
         case OPCCALL:
             {
                 objnum o;
-                
+
+                /* get the argument count */
                 nargc = *p++;
+
+                /* ensure we have enough values to pass as arguments */
+                runcheckargc(ctx, &nargc);
             
                 /* object could move--save offset to restore 'p' after call */
                 o = osrp2(p);
@@ -1361,6 +1365,7 @@ resume_from_error:
         
         case OPCGETP:
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             prop = osrp2(p);
             p += 2;
             obj = runpopobj(ctx);
@@ -1373,8 +1378,7 @@ resume_from_error:
             p += 2;
             obj = runpopobj(ctx);
             runcheckpropdata(ctx, obj, prop);
-            runpprop(ctx, &p, target, targprop, obj, prop, FALSE, nargc,
-                     obj);
+            runpprop(ctx, &p, target, targprop, obj, prop, FALSE, 0, obj);
             break;
 
         case OPCGETDBLCL:
@@ -1554,6 +1558,7 @@ resume_from_error:
                 runsdef *stkp;
 
                 nargc = *p++;
+                runcheckargc(ctx, &nargc);
                 binum = osrp2(p);
                 ofs = runcpsav(ctx, &p, target, targprop);
                 stkp =  ctx->runcxsp - nargc;
@@ -1572,6 +1577,7 @@ resume_from_error:
             
         case OPCPTRCALL:
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             ofs = runcpsav(ctx, &p, target, targprop);
             runfn(ctx, runpopfn(ctx), nargc);
             p = runcprst(ctx, ofs, target, targprop);
@@ -1579,6 +1585,7 @@ resume_from_error:
             
         case OPCINHERIT:
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             prop = osrp2(p);
             p += 2;
             runpprop(ctx, &p, target, targprop, target, prop, TRUE, nargc,
@@ -1587,6 +1594,7 @@ resume_from_error:
 
         case OPCPTRINH:
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             prop = runpopprp(ctx);
             runpprop(ctx, &p, target, targprop, target, prop, TRUE, nargc,
                      self);
@@ -1594,6 +1602,7 @@ resume_from_error:
             
         case OPCPTRGETP:
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             prop = runpopprp(ctx);
             obj = runpopobj(ctx);
             runpprop(ctx, &p, target, targprop, obj, prop, FALSE, nargc,
@@ -1604,13 +1613,13 @@ resume_from_error:
             prop = runpopprp(ctx);
             obj = runpopobj(ctx);
             runcheckpropdata(ctx, obj, prop);
-            runpprop(ctx, &p, target, targprop, obj, prop, FALSE, nargc,
-                     obj);
+            runpprop(ctx, &p, target, targprop, obj, prop, FALSE, 0, obj);
             break;
             
         case OPCEXPINH:
             /* inheritance from explicit superclass */
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             prop = osrp2(p);
             obj = osrp2(p + 2);
             p += 4;
@@ -1628,6 +1637,7 @@ resume_from_error:
 
         case OPCEXPINHPTR:
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             prop = runpopprp(ctx);
             obj = osrp2(p);
             p += 2;
@@ -1693,6 +1703,7 @@ resume_from_error:
             
         case OPCGETPSELF:
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             prop = osrp2(p);
             p += 2;
             runpprop(ctx, &p, target, targprop, self, prop, FALSE, nargc,
@@ -1703,12 +1714,12 @@ resume_from_error:
             prop = osrp2(p);
             p += 2;
             runcheckpropdata(ctx, self, prop);
-            runpprop(ctx, &p, target, targprop, self, prop, FALSE, nargc,
-                     self);
+            runpprop(ctx, &p, target, targprop, self, prop, FALSE, 0, self);
             break;
 
         case OPCGETPPTRSELF:
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             prop = runpopprp(ctx);
             runpprop(ctx, &p, target, targprop, self, prop, FALSE, nargc,
                      self);
@@ -1716,6 +1727,7 @@ resume_from_error:
             
         case OPCGETPOBJ:
             nargc = *p++;
+            runcheckargc(ctx, &nargc);
             obj = osrp2(p);
             prop = osrp2(p+2);
             p += 4;
@@ -1772,6 +1784,10 @@ resume_from_error:
                 p += 2;
                 ex = &ctx->runcxext[fn];
                 
+#if 0
+/*
+ *   External functions are now obsolete - do not attempt to call 
+ */
                 if (!ex->runxptr)
                 {
                     if ((ex->runxptr = os_exfil(ex->runxnam)) == 0)
@@ -1779,6 +1795,10 @@ resume_from_error:
                 }
                 if (os_excall(ex->runxptr, &ux))
                     runsig1(ctx, ERR_EXTRUN, ERRTSTR, ex->runxnam);
+#else
+                /* external functions are obsolete - throw an error */
+                runsig1(ctx, ERR_EXTRUN, ERRTSTR, ex->runxnam);
+#endif
             }
             break;
             
@@ -1936,7 +1956,8 @@ resume_from_error:
                 runsdef  val3;
                 int      asityp;
                 int      asiext;
-                
+                int      lclnum;
+
                 valp = &val;
                 stkval = &val;
 
@@ -1962,6 +1983,11 @@ resume_from_error:
                     i = runpopnum(ctx);
                     lstp = runpoplst(ctx);
                     break;
+
+                case OPCASILCL:
+                    lclnum = runrp2s(p);
+                    p += 2;
+                    break;
                 }
                 
                 if (asityp != OPCASIDIR)
@@ -1970,7 +1996,7 @@ resume_from_error:
                     switch(opc & OPCASIDEST_MASK)
                     {
                     case OPCASILCL:
-                        OSCPYSTRUCT(val, *(ctx->runcxbp + runrp2s(p) - 1));
+                        OSCPYSTRUCT(val, *(ctx->runcxbp + lclnum - 1));
                         break;
                         
                     case OPCASIPRP:
@@ -2002,11 +2028,63 @@ resume_from_error:
                 switch(asityp)
                 {
                 case OPCASIADD:
-                    runadd(ctx, &val, &val2, 2);
+                    if ((opc & OPCASIIND) != 0)
+                    {
+                        runsdef val4;
+
+                        /* 
+                         *   we're adding to an indexed value out of a list -
+                         *   we need to make sure the list is protected from
+                         *   garbage collection, so push it back on the stack
+                         *   while we're working 
+                         */
+                        val4.runstyp = DAT_LIST;
+                        val4.runsv.runsvstr = lstp;
+                        runrepush(ctx, &val4);
+
+                        /* carry out the addition */
+                        runadd(ctx, &val, &val2, 2);
+
+                        /* 
+                         *   in case the list got moved during garbage
+                         *   collection, retrieve it from the stack 
+                         */
+                        lstp = runpoplst(ctx);
+                    }
+                    else
+                    {
+                        /* no list indexing - just carry out the addition */
+                        runadd(ctx, &val, &val2, 2);
+                    }
                     break;
                     
                 case OPCASISUB:
-                    if (!runsub(ctx, &val, &val2, 2)) goto no_assign;
+                    if ((opc & OPCASIIND) != 0)
+                    {
+                        runsdef val4;
+                        int result;
+
+                        /* as with adding, protect the list from GC */
+                        val4.runstyp = DAT_LIST;
+                        val4.runsv.runsvstr = lstp;
+                        runrepush(ctx, &val4);
+
+                        /* carry out the subtraction and note the result */
+                        result = runsub(ctx, &val, &val2, 2);
+
+                        /* recover the list pointer */
+                        lstp = runpoplst(ctx);
+
+                        /* check to see if we have an assignment */
+                        if (!result)
+                            goto no_assign;
+                    }
+                    else
+                    {
+                        /* no list indexing - just do the subtraction */
+                        if (!runsub(ctx, &val, &val2, 2))
+                            goto no_assign;
+                    }
                     break;
                     
                 case OPCASIMUL:
@@ -2136,8 +2214,7 @@ resume_from_error:
                 switch(opc & OPCASIDEST_MASK)
                 {
                 case OPCASILCL:
-                    OSCPYSTRUCT(*(ctx->runcxbp + runrp2s(p) - 1), *valp);
-                    p += 2;
+                    OSCPYSTRUCT(*(ctx->runcxbp + lclnum - 1), *valp);
                     break;
                     
                 case OPCASIPRP:
@@ -2195,7 +2272,6 @@ resume_from_error:
                         ofs = runindofs(ctx, i, lstp);
                         delp = lstp + ofs;        /* ptr to item to replace */
                         delsiz = datsiz(*delp, delp + 1);  /* size of *delp */
-                        remp = lstp + ofs + delsiz + 1;        /* remainder */
                         remsiz = osrp2(lstp) - ofs - delsiz - 1;
                         newsiz = runsiz(valp);          /* size of new item */
                         newtot = osrp2(lstp) + newsiz - delsiz;  /* new tot */
@@ -2208,7 +2284,11 @@ resume_from_error:
                             val3.runstyp = DAT_LIST;
                             val3.runsv.runsvstr = lstp;
                             runhres3(ctx, newtot, 3, &val, &val2, &val3);
+
+                            /* update all of the pointers within lstp */
                             lstp = val3.runsv.runsvstr;
+                            delp = lstp + ofs;
+                            remp = lstp + ofs + delsiz + 1;
                         }
                         memcpy(ctx->runcxhp + 2, lstp + 2, (size_t)(ofs - 2));
                         
@@ -2305,8 +2385,17 @@ done: ;
          /* enter the debugger with the error code */
          dbgssi(ctx->runcxdbg, ctx->runcxlofs, OPCLINE, err, &p);
 
-         /* resume execution */
-         goto resume_from_error;
+         /* check the error again */
+         switch (err)
+         {
+         case ERR_ARGC:
+             /* we can't continue from this - simply return */
+             break;
+
+         default:
+             /* resume execution */
+             goto resume_from_error;
+         }
      }
      ERREND(ctx->runcxerr);
 #endif /* DBG_OFF */
