@@ -35,6 +35,7 @@ Modified
 #include "wchar.h"
 
 #include "os.h"
+#include "t3std.h"
 #include "vmglob.h"
 
 
@@ -305,16 +306,16 @@ protected:
  */
 struct script_stack_entry
 {
-    script_stack_entry(script_stack_entry *enc, int old_more_mode,
-                       osfildef *fp,
-                       int more_mode, int quiet, int event_script)
+    script_stack_entry(script_stack_entry *encp, int old_more,
+                       osfildef *outfp,
+                       int new_more, int is_quiet, int is_event_script)
     {
-        this->enc = enc;
-        this->old_more_mode = old_more_mode;
-        this->fp = fp;
-        this->more_mode = more_mode;
-        this->quiet = quiet;
-        this->event_script = event_script;
+        this->enc = encp;
+        this->old_more_mode = old_more;
+        this->fp = outfp;
+        this->more_mode = new_more;
+        this->quiet = is_quiet;
+        this->event_script = is_event_script;
     }
 
     /* the enclosing stack level */
@@ -479,6 +480,14 @@ public:
     int is_quiet_script() const
         { return (script_sp_ != 0 && script_sp_->quiet); }
 
+    /* is the script in MORE mode? */
+    int is_moremode_script() const
+        { return (script_sp_ != 0 && script_sp_->more_mode); }
+
+    /* is the script an <eventscript> type? */
+    int is_event_script() const
+        { return (script_sp_ != 0 && script_sp_->event_script); }
+
     /* 
      *   Open a script file.  If 'quiet' is true, no output is displayed
      *   while the script is being processed.  If 'script_more_mode' is
@@ -1379,6 +1388,9 @@ protected:
      */
     wchar_t resume_html_parsing(VMG_ wchar_t c,
                                 const char **sp, size_t *slenp);
+
+    /* parse an HTML entity ('&') markup */
+    wchar_t parse_entity(VMG_ wchar_t *ent, const char **sp, size_t *slenp);
     
     /*
      *   Parse the beginning HTML markup.  This is called when we are
