@@ -28,19 +28,24 @@ Modified
 /*
  *   allocate the stack 
  */
-CVmStack::CVmStack(size_t max_depth)
+CVmStack::CVmStack(size_t max_depth, size_t reserve)
 {
     /* 
-     *   Allocate the array of stack elements.  Overallocate by a few
-     *   elements to leave ourselves a little buffer against mild overages -
-     *   for the most part, we count on the compiler to check for proper
-     *   stack usage at entry to each function, but intrinsics sometimes
-     *   push a few elements without checking. 
+     *   Allocate the array of stack elements.  Allocate the requested
+     *   maximum depth plus the requested reserve space.  Overallocate by a
+     *   few elements to leave ourselves a little buffer against mild
+     *   overages - for the most part, we count on the compiler to check for
+     *   proper stack usage at entry to each function, but intrinsics
+     *   sometimes push a few elements without checking.  
      */
-    arr_ = (vm_val_t *)t3malloc(max_depth * sizeof(arr_[0]) + 25);
+    arr_ = (vm_val_t *)t3malloc((max_depth + reserve + 25) * sizeof(arr_[0]));
     
-    /* remember the maximum depth */
+    /* remember the maximum depth and the reserve depth */
     max_depth_ = max_depth;
+    reserve_depth_ = reserve;
+
+    /* the reserve is not yet in use */
+    reserve_in_use_ = FALSE;
 
     /* initialize the stack pointer */
     init();
